library usps_sdk;

import 'package:sexy_api_client/sexy_api_client.dart';

const String _uspsApiUrl = "https://secure.shippingapis.com";
void _errorThrower(String response){
  if(response.contains("<Error>")){
    throw response;
  }
}
class USPSSdk{
  USPSSdk({
    required this.userID,
  });
  final String userID;
  Future<String> validate({
    ///Firm Name
    String firmName = "",
    ///Delivery Address in the destination address. May contain secondary unit designator, such as APT or SUITE, for Accountable mail.)
    String address1 = "",
    ///Delivery Address in the
    ///destination address.
    ///Required for all mail and
    ///packages, however 11-
    ///digit Destination Delivery
    ///Point ZIP+4 Code can be
    ///provided as an alternative
    ///in the Detail 1 Record.
    required String address2,
    ///City name of the
    ///destination address. 
    String city = "",
    ///Two-character state code
    ///of the destination address
    String state = "",
    ///Urbanization.
    ///For Puerto Rico addresses only.
    String urbanization = "",
    ///Destination 5-digit ZIP
    ///Code. Numeric values (0-
    ///9) only. If International, all
    ///zeroes.
    String zip5 = "",
    ///Destination ZIP+4 Numeric
    ///values (0-9) only. If
    ///International, all zeroes.
    ///Default to spaces if not
    ///available.
    String zip4 = "",
  })async{
    String xmlBody = '''
<?xml version="1.0" ?>
<AddressValidateRequest USERID="$userID">
  <Revision>1</Revision>
  <Address ID="0">
    <FirmName>$firmName</FirmName>
    <Address1>$address1</Address1>
    <Address2>$address2</Address2>
    <City>$city</City>
    <State>$state</State>
    <Urbanization>$urbanization</Urbanization>
    <Zip5>$zip5</Zip5>
    <Zip4>$zip4</Zip4>
  </Address>
</AddressValidateRequest>
''';
    String response  = await SexyAPI(
      url: _uspsApiUrl,
      path: "/ShippingAPI.dll",
      parameters: {
        "API" : "Verify",
        "XML" : xmlBody,
      },
    ).get();
    _errorThrower(response);
    return response;
  }
  //ZIP Code Lookup API
  Future<String> zipCodeLookupAPI({
    ///Firm Name
    String firmName = "",
    ///Delivery Address in the destination address. May contain secondary unit designator, such as APT or SUITE, for Accountable mail.)
    String address1 = "",
    ///Delivery Address in the
    ///destination address.
    ///Required for all mail and
    ///packages, however 11-
    ///digit Destination Delivery
    ///Point ZIP+4 Code can be
    ///provided as an alternative
    ///in the Detail 1 Record.
    required String address2,
    ///City name of the
    ///destination address. 
    String city = "",
    ///Two-character state code
    ///of the destination address
    String state = "",
    ///Urbanization.
    ///For Puerto Rico addresses only.
    String urbanization = "",
    ///Destination 5-digit ZIP
    ///Code. Numeric values (0-
    ///9) only. If International, all
    ///zeroes.
    String zip5 = "",
    ///Destination ZIP+4 Numeric
    ///values (0-9) only. If
    ///International, all zeroes.
    ///Default to spaces if not
    ///available.
    String zip4 = "",
  })async{
    String xmlBody = '''
    <?xml version="1.0" ?>
    <ZipCodeLookupRequest USERID="$userID">
      <Revision>1</Revision>
      <Address ID="0">
        <FirmName>$firmName</FirmName>
        <Address1>$address1</Address1>
        <Address2>$address2</Address2>
        <City>$city</City>
        <State>$state</State>
        <Urbanization>$urbanization</Urbanization>
        <Zip5>$zip5</Zip5>
        <Zip4>$zip4</Zip4>
      </Address>
    </ZipCodeLookupRequest>
    ''';
    String response  = await SexyAPI(
      url: _uspsApiUrl,
      path: "/ShippingAPI.dll",
      parameters: {
        "API" : "ZipCodeLookup",
        "XML" : xmlBody,
      },
    ).get();
    _errorThrower(response);
    return response;
  }
  //TODO: CityStateLookup API
  Future<String> cityStateLookup({
    required int zip5,
  })async{
    String xmlBody = '''
    <CityStateLookupRequest USERID="$userID">
      <ZipCode ID='0'>
        <Zip5>$zip5</Zip5>
      </ZipCode>
    </CityStateLookupRequest>
    ''';
    String response  = await SexyAPI(
      url: _uspsApiUrl,
      path: "/ShippingAPI.dll",
      parameters: {
        "API" : "CityStateLookup",
        "XML" : xmlBody,
      },
    ).get();
    _errorThrower(response);
    return response;
  }
  Future<String> trackAndConfirm({
    ///Must be alphanumeric characters.
    required String trackID,
  })async{
    String xmlBody = '''
    <TrackRequest USERID="$userID">
      <TrackID ID="$trackID"></TrackID>
    </TrackRequest>
    ''';
    String response  = await SexyAPI(
      url: _uspsApiUrl,
      path: "/ShippingAPI.dll",
      parameters: {
        "API" : "TrackV2",
        "XML" : xmlBody,
      },
    ).get();
    _errorThrower(response);
    return response;
  }
}